@@ -61,7 +61,7 @@ def getEpisodeName(tvdb_instance, episode):
         # Series was found, use corrected series name
         correctedShowName = show['seriesname']
 
-    if episode.seasonnumber == -1:
+    if isinstance(episode, DatedEpisodeInfo):
         # Date-based episode
         epnames = []
         for cepno in episode.episodenumbers:
@@ -213,6 +213,10 @@ class FileFinder(object):
         """Finds files from startpath, could be called recursively
         """
         allfiles = []
+        if not os.access(startpath, os.R_OK):
+            log().info("Skipping inaccessible path %s" % startpath)
+            return allfiles
+
         for subf in os.listdir(unicode(startpath)):
             if not self._checkExtension(subf):
                 continue
@@ -303,14 +307,6 @@ class FileParser(object):
                         "episodenumberstart and episodenumberend\n\nPattern"
                         "was:\n" + cmatcher.pattern)
 
-                if 'seasonnumber' in namedgroups:
-                    seasonnumber = int(match.group('seasonnumber'))
-                elif 'year' in namedgroups and 'month' in namedgroups and 'day' in namedgroups:
-                    seasonnumber = -1
-                else:
-                    # No season number specified, usually for Anime
-                    seasonnumber = None
-
                 if 'seriesname' in namedgroups:
                     seriesname = match.group('seriesname')
                 else:
@@ -320,11 +316,26 @@ class FileParser(object):
                 if seriesname != None:
                     seriesname = cleanRegexedSeriesName(seriesname)
 
-                episode = EpisodeInfo(
-                    seriesname = seriesname,
-                    seasonnumber = seasonnumber,
-                    episodenumbers = episodenumbers,
-                    filename = self.path)
+                if 'seasonnumber' in namedgroups:
+                    seasonnumber = int(match.group('seasonnumber'))
+
+                    episode = EpisodeInfo(
+                        seriesname = seriesname,
+                        seasonnumber = seasonnumber,
+                        episodenumbers = episodenumbers,
+                        filename = self.path)
+                elif 'year' in namedgroups and 'month' in namedgroups and 'day' in namedgroups:
+                    episode = DatedEpisodeInfo(
+                        seriesname = seriesname,
+                        episodenumbers = episodenumbers,
+                        filename = self.path)
+                else:
+                    # No season number specified, usually for Anime
+                    episode = NoSeasonEpisodeInfo(
+                        seriesname = seriesname,
+                        episodenumbers = episodenumbers,
+                        filename = self.path)
+
                 return episode
         else:
             raise InvalidFilename(self.path)
@@ -386,10 +397,11 @@ def makeValidFilename(value, normalize_unicode = False, windows_safe = False, cu
     # If the filename starts with a . prepend it with an underscore, so it
     # doesn't become hidden.
 
-    # This is done before calling splitext to handle filename of "."
+    # This is done before calling splitext to handle filename of ".", as
     # splitext acts differently in python 2.5 and 2.6 - 2.5 returns ('', '.')
     # and 2.6 returns ('.', ''), so rather than special case '.', this
-    # special-cases all files starting with "." equally (since dotfiles have)
+    # special-cases all files starting with "." equally (since dotfiles have
+    # no extension)
     if value.startswith("."):
         value = "_" + value
 
@@ -450,6 +462,7 @@ def makeValidFilename(value, normalize_unicode = False, windows_safe = False, cu
             new_length = max_len - len(value)
             extension = extension[:new_length]
         else:
+            # File name is longer than extension, truncate filename.
             new_length = max_len - len(extension)
             value = value[:new_length]
 
@@ -467,6 +480,7 @@ def formatEpisodeNumbers(episodenumbers):
 
     return epno
 
+
 class EpisodeInfo(object):
     """Stores information (season, episode number, episode name), and contains
     logic to generate new name
@@ -503,7 +517,19 @@ class EpisodeInfo(object):
     def fullfilename(self):
         return u"%s.%s" % (self.filename, self.extension)
 
-    def generateFilename(self):
+    def sortable_info(self):
+        """Returns a tuple of sortable information
+        """
+        return (self.seriesname, self.seasonnumber, self.episodenumbers)
+
+    def number_string(self):
+        """Used in UI
+        """
+        return "season: %s, episode: %s" % (
+            self.seasonnumber,
+            ", ".join([str(x) for x in self.episodenumbers]))
+
+    def generateFilename(self, lowercase = False):
         """
         Uses the following config options:
         filename_with_episode # Filename when episode name is found
@@ -512,10 +538,7 @@ class EpisodeInfo(object):
         episode_separator # used to join multiple episode numbers
         """
         # Format episode number into string, or a list
-        if self.seasonnumber != -1:
-            epno = formatEpisodeNumbers(self.episodenumbers)
-        else:
-            epno = str(self.episodenumbers[0])
+        epno = formatEpisodeNumbers(self.episodenumbers)
 
         # Data made available to config'd output file format
         if self.extension is None:
@@ -531,12 +554,133 @@ class EpisodeInfo(object):
             'ext': prep_extension}
 
         if self.episodename is None:
-            if self.seasonnumber is None:
-                fname = Config['filename_without_episode_no_season'] % epdata
-            elif self.seasonnumber == -1:
-                fname = Config['filename_with_date_without_episode'] % epdata
-            else:
-                fname = Config['filename_without_episode'] % epdata
+            fname = Config['filename_without_episode'] % epdata
+        else:
+            if isinstance(self.episodename, list):
+                epdata['episodename'] = formatEpisodeName(
+                    self.episodename,
+                    join_with = Config['multiep_join_name_with']
+                )
+            fname = Config['filename_with_episode'] % epdata
+
+        if lowercase or Config['lowercase_filename']:
+            fname = fname.lower()
+
+        return makeValidFilename(
+            fname,
+            normalize_unicode = Config['normalize_unicode_filenames'],
+            windows_safe = Config['windows_safe_filenames'],
+            custom_blacklist = Config['custom_filename_character_blacklist'],
+            replace_with = Config['replace_invalid_characters_with'])
+
+    def __repr__(self):
+        return "<%s: %s>" % (
+            self.__class__.__name__,
+            self.generateFilename())
+
+
+class DatedEpisodeInfo(EpisodeInfo):
+    def __init__(self,
+        seriesname = None,
+        episodenumbers= None,
+        episodename = None,
+        filename = None):
+
+        self.seriesname = seriesname
+        self.episodenumbers = episodenumbers
+        self.episodename = episodename
+        self.fullpath = filename
+
+    def sortable_info(self):
+        """Returns a tuple of sortable information
+        """
+        return (self.seriesname, self.episodenumbers)
+
+    def number_string(self):
+        """Used in UI
+        """
+        return "episode: %s" % (
+            ", ".join([str(x) for x in self.episodenumbers]))
+
+    def generateFilename(self, lowercase = False):
+        # Format episode number into string, or a list
+        dates = str(self.episodenumbers[0])
+        if isinstance(self.episodename, list):
+            prep_episodename = formatEpisodeName(
+                self.episodename,
+                join_with = Config['multiep_join_name_with']
+            )
+        else:
+            prep_episodename = self.episodename
+
+        # Data made available to config'd output file format
+        if self.extension is None:
+            prep_extension = ''
+        else:
+            prep_extension = '.%s' % self.extension
+
+        epdata = {
+            'seriesname': self.seriesname,
+            'episode': dates,
+            'episodename': prep_episodename,
+            'ext': prep_extension}
+
+        if self.episodename is None:
+            fname = Config['filename_with_date_without_episode'] % epdata
+        else:
+            fname = Config['filename_with_date_and_episode'] % epdata
+
+        if lowercase or Config['lowercase_filename']:
+            fname = fname.lower()
+
+        return makeValidFilename(
+            fname,
+            normalize_unicode = Config['normalize_unicode_filenames'],
+            windows_safe = Config['windows_safe_filenames'],
+            custom_blacklist = Config['custom_filename_character_blacklist'],
+            replace_with = Config['replace_invalid_characters_with'])
+
+
+class NoSeasonEpisodeInfo(EpisodeInfo):
+    def __init__(self,
+        seriesname = None,
+        episodenumbers= None,
+        episodename = None,
+        filename = None):
+
+        self.seriesname = seriesname
+        self.episodenumbers = episodenumbers
+        self.episodename = episodename
+        self.fullpath = filename
+
+    def sortable_info(self):
+        """Returns a tuple of sortable information
+        """
+        return (self.seriesname, self.episodenumbers)
+
+    def number_string(self):
+        """Used in UI
+        """
+        return "episode: %s" % (
+            ", ".join([str(x) for x in self.episodenumbers]))
+
+    def generateFilename(self, lowercase = False):
+        epno = formatEpisodeNumbers(self.episodenumbers)
+
+        # Data made available to config'd output file format
+        if self.extension is None:
+            prep_extension = ''
+        else:
+            prep_extension = '.%s' % self.extension
+
+        epdata = {
+            'seriesname': self.seriesname,
+            'episode': epno,
+            'episodename': self.episodename,
+            'ext': prep_extension}
+
+        if self.episodename is None:
+            fname = Config['filename_without_episode_no_season'] % epdata
         else:
             if isinstance(self.episodename, list):
                 epdata['episodename'] = formatEpisodeName(
@@ -544,24 +688,18 @@ class EpisodeInfo(object):
                     join_with = Config['multiep_join_name_with']
                 )
 
-            if self.seasonnumber is None:
-                fname = Config['filename_with_episode_no_season'] % epdata
-            elif self.seasonnumber == -1:
-                fname = Config['filename_with_date_and_episode'] % epdata
-            else:
-                fname = Config['filename_with_episode'] % epdata
+            fname = Config['filename_with_episode_no_season'] % epdata
+
+        if lowercase or Config['lowercase_filename']:
+            fname = fname.lower()
 
         return makeValidFilename(
             fname,
             normalize_unicode = Config['normalize_unicode_filenames'],
             windows_safe = Config['windows_safe_filenames'],
+            custom_blacklist = Config['custom_filename_character_blacklist'],
             replace_with = Config['replace_invalid_characters_with'])
 
-    def __repr__(self):
-        return "<%s: %s>" % (
-            self.__class__.__name__,
-            self.generateFilename())
-
 
 def same_partition(f1, f2):
     """Returns True if both files or directories are on the same partition
